#include <stdio.h>
#include <string.h>
#include <sys/timeb.h>
#include "mparser.h"

#ifdef __WIN32__
#include <windows.h>
#define ftime _ftime
#endif /* __WIN32__ */


/* multiarg test function  */
char* dummy( int paramcnt, double *args, hqStrMap *strparams, double *result );

char expr[] =
    "dummy(5) + CoS(PI+4) - round(25,-1) + LN(E**neG(2.5)) - (3/2) ^((1*2/3/4/*5*/))";
char funcnames[] =
    "DUMMY\000";
char* (*funcaddrs[])() = {
    &dummy };

hqStrMap* strmap;

const int repeats = 8;

char buffer[256];

char *translate( char *str )
{
#ifdef __WIN32__
    CharToOem( str, buffer );
    return buffer;
#else
    return str;
#endif
}
            
int main( int argc, char **argv )
{
    char *ErrMsg;
    hqMathParser *parser;
    double result;
    if (argc < 2) {
	printf( "Usage: ccalc <expression>\n" );
	printf( "    or ccalc --test\n" );
	return 1;
    }
    strmap = Strmap_CreateFromChain( sizeof(void*), funcnames, funcaddrs );
    parser = MathParser_Create( NULL /*moreletters*/ );
    parser->ExtFunctions = strmap;

    if (strcmp( argv[1], "--test" )==0)
    {
	double cps;
	int i, msec;
	long time1, time2;
	struct timeb tb;
	ftime( (void*) &tb );
	time1 = tb.time * 1000 + tb.millitm;
        for (i=0; i<0xFFF*repeats; i++) {
	    ErrMsg = MathParser_Parse( parser, expr, &result );
	    if ( ErrMsg == NULL ) {
		if ((i & 0xFFF) == 0xFFF) {
	    	    printf( "." );
		    fflush( stdout );
		}
	    } else {
		puts( translate( ErrMsg ) );
	    	return 1;
	    }
        }
	ftime( (void*) &tb );
	time2 = tb.time * 1000 + tb.millitm;
	msec = (int)(time2 - time1);
	cps = (msec==0) ? -1 : (double)(0xFFF*repeats)*1000/(double)msec;
	printf( "\n%d calculations / %d millisec; %G c/s\n",
		0xFFF*repeats, msec, cps );
    } else {
    	ErrMsg = MathParser_Parse( parser, argv[1], &result );
    	if ( ErrMsg == NULL )
	    printf( "%.15G\n", result );
    	else {
	    puts( translate( ErrMsg ) );
	    return 1;
	}
    }
    MathParser_Destroy( parser );
    StrMap_Destroy( strmap );
    return 0;
}

/* this function is completely for test purposes */

char* dummy( int paramcnt, double *args, hqStrMap *strparams, double *result )
{
    if (paramcnt>1)
	return "#Invalid parameters count!";

    if (paramcnt==0) {
	*result = 0.0;
	return NULL;
    }
    if (*args==DblNiN) { 
        int len;
	char *str = StrMap_GetString( strparams, 0, &len, NULL );
        fwrite( str, 1, len, stdout );
	puts("\n");
	return NULL;
    }
    *result = *args;
    return NULL;
}
